#ifndef labbCAN_Pump_APIH
#define labbCAN_Pump_APIH
//===========================================================================
// CETONI GmbH
// Gewerbegebiet Korbwiesen
// Wiesenring 6
// 07554 Korbussen
//
// (c) Copyright 2017, CETONI GmbH
//
// All rights reserved. Cetoni's source code is an unpublished work
// and the use of a copyright notice does not imply otherwise. This
// source code contains confidential, trade secret material of
// cetoni GmbH. Any attempt or participation in deciphering, decoding,
// reverse engineering or in any way altering the source code is
// strictly prohibited, unless the prior written consent cetoni GmbH
// is obtained.
//
/// \file     labbCAN_Pump_API.h
/// \author   Uwe Kindler (UK)
/// \date     2012/07/13
/// \brief    labbCAN Pump Application Programming Interface
//
//===========================================================================

/**
 * \page PumpAPI Pump Library
  * \ifnot bootstrap
 *    \tableofcontents
 * \endif
 * \image html Slider_Pump_API_color.jpg
 * \section PumpAPI_Introduction Introduction
 * \subsection PumpAPI_General General
 * This pump library defines a common pump control interface for a
 * range of different pump types such as e.g. syringe pumps, dilutors,
 * peristaltic pumps, or diaphragm pumps.
 *
 * In general a pump consists of an actuator (like a linear drive for a syringe
 * pump, a servo drive for a peristaltic pump or a simple DC/EC drive for a
 * diaphragm pump) and a container for the transport of the fluid (like a
 * syringe for a syringe pump/dilutor, a tube for a peristaltic pump or a
 * pump head for the diaphragm pump). With some pumps like syringe pumps and
 * peristaltic pumps the container is exchangeable by the application engineer
 * or even by the user of the laboratory device. That means a user may mount
 * different kind of syringes on a syringe pump or may use different tube sizes
 * on a peristaltic pump. With other pumps (i.e. diaphragm pumps) the container
 * and the actuator are fixed to each other and build up one single unit. The
 * pump parameters (e.g. volume, flow or pressure) may be limited by the
 * actuator and/or by the container.
 *
 * \subsection PumpAPI_ActuatorLimits Actuator limits
 * On a syringe pump the flow is limited by the speed of the actuator
 * (the velocity of the linear drive) and the volume is limited by the
 * length/maximum position of the linear drive. On a peristaltic pump or
 * diaphragm pump the volume is not limited by the actuator but the maximum
 * flow is limited by the actuator speed - the maximum rotation speed of the
 * peristaltic pump drive or the speed of the DC/EC diaphragm pump drive.
 *
 * \subsection PumpAPI_ContainerLimits Container limits
 * Container limits are only important for devices where it is possible to
 * change the container. On every change of the container the calculation of
 * the flow and volume values changes and therefore the parameters maximum
 * volume and maximum flow may also change. That means each container has its
 * own container limits. The maximum volume of a syringe pump is limited by
 * the stroke of the syringe piston and the inner syringe diameter. The maximum
 * flow is limited by the inner diameter of the syringe. The tube dimensions do
 * not limit the maximum volume of a peristaltic pump but the maximum flow is
 * limited by the inner diameter of the tube.
 *
 * ---
 *
 * \section PumpAPI_Including Including
 * The way how you include the library functions in your own windows program,
 * depends on the compiler and on the programming language you use. In order to
 * access the pump API, you have to include the library
 * **labbCAN_Pump_API.dll** to your programming environment. You have
 * to copy this file to the working directory of your system or to the
 * application directory that contains your application EXE file.
 * Use the calling convention \b LCP_CALL for this library. This convention
 * is managing how the parameters are put on the stack and who is responsible
 * to clean the stack after the function execution.
 *
 * The interface of the library, constant definitions and declarations of the
 * library functions, is defined in the header file
 * \link labbCAN_Pump_API.h interface/labbCAN_Pump_API.h \endlink
 *
 * ---
 *
 * \section PumpAPI_Init Initialization
 * \subsection PumpAPI_Step1 Step 1 - Retrieve device handle
 * To work with a pump, you need a valid device handle. A device handle is
 * simply an opaque pointer to the internal device object created by the
 * labbCAN environment. So the first thing you need do to is obtaining a valid
 * device handle.
 *
 * You can get a valid pump handle with the functions
 * LCP_LookupPumpByName() or LCP_GetPumpHandle():
 * \code
 * dev_hdl PumpHandle;
 * Result = LCP_LookupPumpByName("Nemesys1_Pump", &PumpHandle);
 * dev_hdl PumpHandle2;
 * Result = LCP_GetPumpHandle(1, &PumpHandle2);
 * \endcode
 *
 * \note You can get the pump names from the device configuration files (see
 * \ref ConfigFiles)
 *
 * \subsection PumpAPI_Step2 Step 2 - Enable devices
 * If you have valid device handle, you can start to bring your devices into a
 * properly enabled and initialized state. To do this, you need to:
 * 1. clear all faults (LCP_ClearFault())
 * 2. enable all pump drives (LCP_Enable())
 * 3. execute a reference move (if required) to find the device homing positions
 * (LCP_SyringePumpCalibrate()) or restore the previously saved drive position
 * counter via LCP_RestoreDrivePosCnt()
 *
 * \subsection PumpAPI_InitExamples Examples
 * The following example code snippet shows a valid valid pump initialization
 * procedure:
 * \includelineno Pump_CAPI_Init.cpp
 *
 * \subsection PumpAPI_InitReference Reference
 * See the \ref lcp_api_init module and the \ref lcp_cfg module for a detailed
 * reference of all pump initialization and configuration functions.
 *
 * ---
 *
 * \section PumpAPI_Dosing Dosing
 * For almost all dosing tasks the functions from the \ref lcp_control module
 * and from the \ref lcp_status module are required. Using the pump API you can
 * choose from a number of different dosing modes:
 * - \ref LCP_Aspirate() for the aspiration of a certain volume with a certain flow
 * - \ref LCP_Dispense() for dispension of a certain volume
 * - \ref LCP_PumpVolume() for aspiration or dispension depending on the sign
 *   of the flow rate parameter (negativ = aspiration, positive = dispension)
 * - \ref LCP_SetFillLevel() aspirates or dispenses until a certain syringe
 *   fill level is reached
 * - \ref LCP_GenerateFlow() generates a constant flow - dosing continues until
 *   it gets stopped manually or until pusher reached one of its limits.
 *
 * \note All dosing functions are non blocking. That means each function returns
 * Immediately after the dosing process has been started and you need to poll the
 * LCP_IsPumping() function to detect if dosing has been finished.
 *
 * With the status functions you can always query the actual state of the
 * pumps you are working with:
 * - LCP_GetDosedVolume() returns the dosed volume since start of last dosage
 * - LCP_GetFillLevel()returns the actual fill level of the pump container (eg.syringe)
 * - LCP_GetFlowIs() always returns the actual flow rate of the pump
 * - LCP_IsPumping() returns true (1) as long as dosing is active.
 *
 * Each pump drive tracks the actual position value (volume value) by an
 * internal position counter. If the pumps are switched off, the actual value
 * of the position counter gets lost and is initialized to zero at next start.
 * So right after power on, you don't know anything about the actual position
 * of the pump drive units. For pumps like peristaltic pumps this does not matter
 * but for pumps like syringe pumps this is very important because the position
 * defines the actual fill level.
 *
 * To properly initialize the internal position counter you either need to
 * execute a reference move via LCP_SyringePumpCalibrate() or you need to
 * restore a previously saved position counter value via LCP_RestoreDrivePosCnt().
 * Because a safe reference move is possible only if there is now syringe
 * mounted on the device, the recommended way is using LCP_RestoreDrivePosCnt().
 * That means, right before you shut down your system, you need to query the
 * actual position counter value of each pump via LCP_GetDrivePosCnt(). Then
 * you need to store this value into a file. The next time you start up your
 * devices, you load the saved position values, and write it back into the
 * devices vial LCP_RestoreDrivePosCnt().
 *
 * \warning If you manually change the position of the pusher of you syringe
 * pump while it is switched of, the saved position counter value ist not valid
 * anymore and you need to execute a calibration move via
 * LCP_SyringePumpCalibrate().
 *
 * \subsection PumpAPI_DosingExamples Examples
 * The following example code snippet shows how to empty a syringe completely
 * and then aspirate a certain volume with a certain flow rate.
 * \includelineno Pump_CAPI_Dosing.cpp
 *
 * \subsection PumpAPI_DosingReference Reference
 * See the \ref lcp_control module and the \ref lcp_status module for a detailed
 * reference of all pump dosing and status functions.
 *
 * ---
 *
 * \section PumpAPI_ProgrammingInterface Programming Interface - API
 * See the \ref lcp_api module for a detailed reference of
 * the pump library application programming interface
 *
 * \example Pump_CAPI_Init.cpp Pump initialization
 * The following example shows how to get a valid pump handle from the pump
 * library and how to properly setup and configure pump parameters.
 *
 * \example capi_nemesys_test.cpp neMESYS pump C-API test
 * The following sample source code shows the implementation of the neMESYS
 * C-API test. The test uses almost all \ref PumpAPI functions and shows how to
 * use them. It further shows how to get a valve device handle from neMESYS
 * pump device and how to control neMESYS valves via \ref ValveAPI.
 *
 */

/**
 * \addtogroup lcp_api labbCAN Pump API
 * \brief Pump specific functions for various pump types.
 *
 * <img src="MenuItem_Pump_Library.png" align="left" style="padding-right: 10px;">
 * This module defines common labbCAN pump specific functions, identifiers
 * and data types for driving labbCAN conform pumps (syringe pumps, tubing pumps)
 * that are attached to a labbCAN software bus.
 *  \see See \ref PumpAPI for a detailed introduction and general overview of
 *  labbCAN pump library.
 *
 *  <div style="padding: 40px;"></div>
 * \{
 */

//===========================================================================
//                                 INCLUDES
//===========================================================================
#include "labbCAN_Bus_API.h"


//===========================================================================
//                                DEFINES
//===========================================================================
/// \cond private
#ifndef LCP_Func
	#define LCP_Func QMIXSDK_EXTERNC QMIXSDK_DLLIMPORT
#endif
#define LCP_CALLBACK QMIXSDK_CALLBACK
#define LCP_CALL QMIXSDK_CALL
/// \endcond


/// \addtogroup lcp_pumpunits Pump SI units
/// Defines common SI units for volume and flow rate for all pump devices.
/// \{
//---------------------------------------------------------------------------
/// \addtogroup lcp_volunit Volume SI Units
/// Defines common volume SI units for all pump devices.
/// The units are based on the standardized CANopen unit system
/// \{
#define LITRES 68 //!< LITRES
/// \} lcp_volunit


//---------------------------------------------------------------------------
/// \addtogroup lcp_prefix Prefix Constants
/// Prefix constants for SI volume and flow rate unit specification
/// \{
#define UNIT   0 //!< use base unit
#define DECI  -1 //!< DECI
#define CENTI -2 //!< CENTI
#define MILLI -3 //!< MILLI
#define MICRO -6 //!< MICRO
/// \} lcp_prefix


//---------------------------------------------------------------------------
/// \addtogroup lcp_timeunit Time Units
/// Time unit identifies for flow rates
/// \{
#define PER_SECOND    1 //!< PER_SECOND
#define PER_MINUTE   60 //!< PER_MINUTE
#define PER_HOUR   3600 //!< PER_HOUR
/// \} lcp_timeunit
/// \} lcp_pumpunits


//===========================================================================
//                     PUMP API INITIALISATION
//
/// \addtogroup lcp_api_init Pump API Initialisation
/// API specific initialisation functions
/// This groups defines all functions to setup and initialize the labbCAN
/// pump API properly.
/// \{
//===========================================================================

//---------------------------------------------------------------------------
/// Query number of detected pump devices
/// \retval >= 0 Number of detected pump devices
/// \retval < 0  Error code
///
LCP_Func long LCP_CALL LCP_GetNoOfPumps();


//---------------------------------------------------------------------------
/// Lookup for a pump device by its name.
/// \param[in] pPumpName The name of the device to lookup for
/// \param[out] PumpHandle Handle to pump device or 0 if no pump with given
///             name exists
/// \retval -ERR_NODEV No device with the given name found
/// \retval ERR_NOERR indicates success
///
LCP_Func long LCP_CALL LCP_LookupPumpByName(const char *pPumpName,
	dev_hdl* PumpHandle);


//---------------------------------------------------------------------------
/// Get pump handle by its index.
/// \param[in] Index Index into list of available pump devices
/// \param[out] PumpHandle Handle to pump device or 0 if the given index is
///             out of range
/// \retval -ERR_PARAM_RANGE If the given device index is out of range
/// \retval ERR_NOERR indicates success
///
LCP_Func long LCP_CALL LCP_GetPumpHandle(unsigned char Index,
	dev_hdl* PumpHandle);
/// \} lcp_api_init



//===========================================================================
//                     PUMP CONFIGURATION
//
/// \addtogroup lcp_cfg Pump Configuration
/// Functions for configuration of pump parameters and pump SI units.
///
/// \{
//===========================================================================

//---------------------------------------------------------------------------
/// This function sets the default volume unit.
/// All parameters of subsequent dosing function calls are given in this new
/// unit. Example to select milliliters as volume unit. Example:
/// \code
/// long Result = LCP_SetVolumeUnit(hPump1, MILLI, LITRES);
/// \endcode
/// \param[in] hPump A valid syringe pump handle
/// \param[in] Prefix The prefix of the SIunit (i.e MILLI - see @ref lcp_prefix)
/// \param[in] VolumeUnit The volume unit identifier (i.e. LITRES - see @ref
///            lcp_volunit)
/// \return Error code - ERR_NOERR indicates success
/// \see LCP_GetVolumeUnit()
///
LCP_Func long LCP_CALL LCP_SetVolumeUnit(dev_hdl hPump,
                                          int    Prefix,
                                          int     VolumeUnit);


//---------------------------------------------------------------------------
/// Queries the current volume unit used for all dosage functions.
/// \param[in] hPump A valid syringe pump handle
/// \param[out] pPrefix Returns the prefix of the SIunit
///             (i.e MILLI - see @ref lcp_prefix)
/// \param[out] pVolumeUnit Returns the volume unit identifier
///            (i.e. LITRES - see @ref lcp_volunit)
/// \return Error code - ERR_NOERR indicates success
/// \see LCP_SetVolumeUnit()
///
LCP_Func long LCP_CALL LCP_GetVolumeUnit(dev_hdl hPump,
	                                      int*   pPrefix,
                                          int*    pVolumeUnit);


//---------------------------------------------------------------------------
/// Sets the flow unit for a certain pump.
/// The flow unit defines the unit to be used for all flow values passed
/// to API functions or retrieved from API functions.
/// \code
/// long Result = LCA_SetFlowUnit(hAxis, MILLI, LITRES, PER_SECOND);
/// \endcode
/// \param[in] hPump A valid syringe pump handle
/// \param[in] Prefix The prefix of the velocity unit (see @ref lcp_prefix)
/// \param[in] VolumeUnit The volume unit (numerator) of the velocity
///            unit (i.e. LITRES - see @ref lcp_volunit)
/// \param[in] TimeUnit The time unit (denominator) of the velocity unit.
///            (i.e. PER_SECOND - see @ref lcp_timeunit)
/// \return Error code - ERR_NOERR indicates success
/// \see LCP_GetFlowUnit()
///
LCP_Func long LCP_CALL LCP_SetFlowUnit(dev_hdl hPump,
                                        int    Prefix,
                                        int     VolumeUnit,
                                        int     TimeUnit);


//---------------------------------------------------------------------------
/// Queries the current flow unit used for passing flow values.
/// \param[in] hPump A valid syringe pump handle
/// \param[out] pPrefix Returns the prefix of the flow unit (see @ref lcp_prefix)
/// \param[out] pVolumeUnit Returns the volume unit (numerator) of the
///             velocity unit (i.e. LITRES - see @ref lcp_volunit)
/// \param[out] pTimeUnit Returns the time unit (denominator) of the flow
///             unit. (i.e. PER_SECOND - see @ref lcp_timeunit)
/// \return Error code - ERR_NOERR indicates success
/// \see LCP_SetFlowUnit()
///
LCP_Func long LCP_CALL LCP_GetFlowUnit(dev_hdl hPump,
                                        int*   pPrefix,
                                        int*    pVolumeUnit,
                                        int*    pTimeUnit);


//---------------------------------------------------------------------------
/// Get maximum flow rate that is realizable with current dosing unit configuration.
/// The maximum flow rate depends on the mechanical configuration of the
/// dosing unit (gear) and on the syringe configuration. If larger syringes
/// are used then larger flow rates are realizable.
/// \param[in] hPump A valid syringe pump handle
/// \param[out] FlowRateMax Returns the maximum flow rate in configured SI unit
/// \return Error code - ERR_NOERR indicates success
///
LCP_Func long LCP_CALL LCP_GetFlowRateMax(dev_hdl hPump, double* FlowRateMax);


//===========================================================================
//                     SYRINGE PUMP CONFIGURATION
//
/// \addtogroup lcp_syringe_cfg Syringe Pump Configuration
/// Syringe pump specific functions for configuration.
/// This group defines functions for configuration of syringe pump parameters
/// like configuration of syringe parameters.
///
/// \{
//===========================================================================


//---------------------------------------------------------------------------
/// Read syringe parameters.
/// \param[in] hPump A valid syringe pump handle
/// \param[out] pInnerDiameter_mm Inner diameter of the syringe tube in
///             Millimeters.
/// \param[out] pMaxPistonStroke_mm The maximum piston stroke defines the
///                                 maximum position the piston can be moved
///                                 to before it slips out of the syringe tube.
///                                 The maximum piston stroke limits the maximum
///                                 travel range of the syringe pump pusher.
/// \return Error code - ERR_NOERR indicates success
///
LCP_Func long LCP_CALL LCP_GetSyringeParam(dev_hdl hPump,
                                            double *pInnerDiameter_mm,
                                            double *pMaxPistonStroke_mm);


//---------------------------------------------------------------------------
/// Set syringe parameters.
/// If you change the syringe in one device, you need to setup the new
/// syringe parameters to get proper conversion of flow rate und volume
/// units.
/// \param[in] hPump A valid syringe pump handle
/// \param[in]  InnerDiameter_mm  Inner diameter of the syringe tube in
///                               millimetres.
/// \param[in]  MaxPistonStroke_mm  The maximum piston stroke defines the
///                                 maximum position the piston can be moved
///                                 to before it slips out of the syringe tube.
///                                 The maximum piston stroke limits the maximum
///                                 travel range of the syringe pump pusher.
/// \return Error code - ERR_NOERR indicates success
///
LCP_Func long LCP_CALL LCP_SetSyringeParam(dev_hdl hPump,
                                            double  InnerDiameter_mm,
                                            double  MaxPistonStroke_mm);


//---------------------------------------------------------------------------
/// Returns the maximum volume a pump can aspirate into its container
/// (syringe)
/// For some devices like peristaltic pumps the maximum volume is not limited
/// while for other pumps, like syringe pumps, the maximum volume is limited
/// by the maximum travel range of the drive unit and by the maximum piston
/// stroke of the mounted syringe. This function returns the maximum volume
/// the pump can take up with the current container (eg.syringe) configuration
/// \param[in] hPump A valid syringe pump handle
/// \param[out] VolumeMax Returns the maximum volume in configured SI units
/// \return Error code - ERR_NOERR indicates success
///
LCP_Func long LCP_CALL LCP_GetVolumeMax(dev_hdl hPump, double* VolumeMax);

/// \} lcp_syringe_cfg
/// \} lcp_cfg


//===========================================================================
//                             PUMP CONTROL
//
/// \addtogroup lcp_control Pump Control
/// Functions to control the pump parameters like flow and volume.
/// This groups defines all functions to control the pump and to control
/// simple dosing tasks like dosing volumes and generating constant flow.
/// \{
//===========================================================================

//---------------------------------------------------------------------------
/// Executes a reference move for a syringe pump.
/// \warning A reference move will move the pusher slightly below the zero
/// position. This may cause a damage to a mounted glass syringe. So if you use
/// this function in your software, you should warn the user about this
/// risk and ask him to unmount the syringe.
/// \param[in] hPump A valid pump handle
/// \return Error code - ERR_NOERR indicates success
/// \see LCP_IsCalibrationFinished()
///
LCP_Func long LCP_CALL LCP_SyringePumpCalibrate(dev_hdl hPump);


//---------------------------------------------------------------------------
/// Pumps fluid with the given flow rate until the requested fill level is
/// reached. Depending on the requested fill level given in Level parameter this
/// function may cause aspiration or dispension of fluid. This function only
/// works properly for pump devices that support a fill level (eg. syringe
/// pumps). Pumps like peristaltic pumps do not support a fill level and the
/// function returns an error for unsupported pump types.
/// \param[in] hPump A valid pump handle
/// \param[in,out] Level The requested fill level. A level of 0 indicates a
///                completely empty container (eg. empty syringe).
/// \param[in,out] Flow The flow rate to use for pumping
/// \retval ERR_NOERR indicates success
/// \retval -ERR_PARAM_RANGE Level or Flow value is out of range
///
LCP_Func long LCP_CALL LCP_SetFillLevel(dev_hdl hPump,
	                                     double  Level,
	                                     double  Flow);


//---------------------------------------------------------------------------
/// Pump a certain volume with a certain flow rate.
/// \param[in] hPump A valid pump handle
/// \param[in] Volume The volume to pump in physical units
///                   negative volumes indicate aspiration
///                   positive volumes indicate dispensing
/// \param[in] Flow   The flow rate to use to pump the volume
/// \retval ERR_NOERR indicates success
/// \retval -ERR_PARAM_RANGE Level or Flow value is out of range
///
LCP_Func long LCP_CALL LCP_PumpVolume(dev_hdl hPump, double Volume, double Flow);


//--------------------------------------------------------------------------
/// Dispense a certain volume with a certain flow rate.
/// \param[in] hPump A valid pump handle
/// \param[in] Volume The volume to dispense in physical units
/// \param[in] Flow   The flow rate to use to dispense the volume
///                   negative flow rates are invalid
/// \retval ERR_NOERR indicates success
/// \retval -ERR_PARAM_RANGE Level or Flow value is out of range
///
LCP_Func long LCP_CALL LCP_Dispense(dev_hdl hPump, double Volume, double Flow);


//--------------------------------------------------------------------------
/// Aspirate a certain volume with a certain flow rate.
/// \param[in] hPump A valid pump handle
/// \param[in] Volume The volume to aspirate in physical units
/// \param[in] Flow   The flow rate to use to aspirate the volume
///                   negative flow rates are invalid
/// \retval ERR_NOERR indicates success
/// \retval -ERR_PARAM_RANGE Level or Flow value is out of ranges
///
LCP_Func long LCP_CALL LCP_Aspirate(dev_hdl hPump, double Volume, double Flow);


//--------------------------------------------------------------------------
/// Generate a continuous flow.
/// \param[in] hPump A valid pump handle
/// \param[in] FlowRate A positive flow rate indicates dispensing
///                     and a negative flow rate indicates aspiration
/// \retval ERR_NOERR indicates success
/// \retval -ERR_PARAM_RANGE Flow value is out of range
///
LCP_Func long LCP_CALL LCP_GenerateFlow(dev_hdl hPump, double FlowRate);


//--------------------------------------------------------------------------
/// Immediately stop pumping.
/// \param[in] hPump A valid pump handle
/// \return Error code - ERR_NOERR indicates success
///
LCP_Func long LCP_CALL LCP_StopPumping(dev_hdl hPump);


//--------------------------------------------------------------------------
/// Immediately stop pumping off all pumps.
/// \return Error code - ERR_NOERR indicates success
///
LCP_Func long LCP_CALL LCP_StopAllPumps();
/// \} lcp_control


//===========================================================================
//                             PUMP STATUS
//
/// \addtogroup lcp_status Pump Status
/// Functions to query the actual pump status, flow and volume.
/// This groups defines all functions to read the actual pump status
/// (pumping or stopped), the actual flow and volume from the device.
/// \{
//===========================================================================

//--------------------------------------------------------------------------
/// Read the actual flow rate.
/// This does not read the real flow rate instead this function returns
/// simply the cached flow rate demand value
/// \param[in] hPump A valid pump handle
/// \param[out] pFlowRateIs The actual flow rate demand value
/// \return Error code - ERR_NOERR indicates success
///
LCP_Func long LCP_CALL LCP_GetFlowIs(dev_hdl hPump, double *pFlowRateIs);


//--------------------------------------------------------------------------
/// Read the target volume.
/// This function simply returns the set target volume value
/// \param[in] hPump A valid pump handle
/// \param[out] pTargetVolume Returns the target volume to dose
/// \return Error code - ERR_NOERR indicates success
///
LCP_Func long LCP_CALL LCP_GetTargetVolume(dev_hdl hPump, double *pTargetVolume);


//--------------------------------------------------------------------------
/// Get the already dosed volume.
/// \param[in] hPump A valid pump handle
/// \param[out] pDosedVolume Returns the already dosed volume
/// \return Error code - ERR_NOERR indicates success
///
LCP_Func long LCP_CALL LCP_GetDosedVolume(dev_hdl hPump, double *pDosedVolume);

//--------------------------------------------------------------------------
/// Returns the actual fill level of the pump.
/// This function returns valid results only for pumps that support a fill level
/// (eg. syringe pumps). Peristaltic pumps do not support fill level.
/// For a syringe pump this function returns the current syringe fill level
/// \param[in] hPump A valid pump handle
/// \param[out] pFillLevel Returns the current fill level of the pump container
///                        (eg. syringe)
/// \return Error code - ERR_NOERR indicates success
///
LCP_Func long LCP_CALL LCP_GetFillLevel(dev_hdl hPump, double* pFillLevel);


//--------------------------------------------------------------------------
/// Check if device is currently stopped or dosing.
/// \param[in] hPump A valid pump handle
/// \retval 1 - Device is pumping
/// \retval 0 - Device is stopped
/// \retval <0 - Error code
///
LCP_Func long LCP_CALL LCP_IsPumping(dev_hdl hPump);


//---------------------------------------------------------------------------
/// Checks if calibration is finished.
/// The pump handle given in hPump param needs to be a syringe pump handle
/// \param[in] hPump A valid pump handle
/// \retval 0 - Device is calibrating
/// \retval 1 - Device calibration has finished
/// \retval <0 - Error code
/// \see LCP_SyringePumpCalibrate()
///
LCP_Func long  LCP_CALL LCP_IsCalibrationFinished(dev_hdl hPump);
/// \} lcp_status


//===========================================================================
//                            PUMP DRIVE
//
/// \addtogroup lcp_drv Pump Drive Control
/// Functions to control and maintain the drive that drives the pump.
/// \{
//===========================================================================

//--------------------------------------------------------------------------
/// Query if pump drive is enabled.
/// Only if the pump drive is enabled it is possible to pump fluid
/// \param[in] hPump A valid pump handle
/// \retval 1 - Pump drive is enabled, pumping is possible
/// \retval 0 - Pump drive is disabled - pump head is free running
/// \retval <0 - Error code
///
LCP_Func long LCP_CALL LCP_IsEnabled(dev_hdl hPump);


//--------------------------------------------------------------------------
/// Check if pump is in a fault state.
/// If the device is in fault state then it is necessary to call
/// LCP_ClearFault() to clear the fault state and then LCP_Enable()
/// to enable the pump drive
/// \param[in] hPump A valid pump handle
/// \retval 1 - Pump is in fault state
/// \retval 0 - Pump is not in fault state
/// \retval <0 - Error code - Error reading the fault state
///
LCP_Func long LCP_CALL LCP_IsInFaultState(dev_hdl hPump);


//---------------------------------------------------------------------------
/// Clear fault condition.
/// This is some kind of error acknowledge that clears the last fault and
/// sets the device in an error free state. If the function
/// LCP_IsInFaultState(void) indicates that device is in fault state, then
/// this function may clear the fault. If the device is still in fault state
/// after this function was called then a serious failure occurred
/// \param[in] hPump A valid pump handle
/// \return Error code
///
LCP_Func long LCP_CALL LCP_ClearFault(dev_hdl hPump);


//--------------------------------------------------------------------------
/// Set axis into enabled state.
/// If the function LCP_IsEnabled(void) indicates that the device is
/// disabled then call this function.
/// \param[in] hPump A valid pump handle
/// \return Error code
///
LCP_Func long LCP_CALL LCP_Enable(dev_hdl hPump);


//--------------------------------------------------------------------------
/// Set pump into disabled state.
/// Call this function to remove power from pump drive
/// \param[in] hPump A valid pump handle
/// \return Error code
///
LCP_Func long LCP_CALL LCP_Disable(dev_hdl hPump);


//--------------------------------------------------------------------------
/// Returns the drive handle of pump drive for the given pump.
/// You can pass the drive handle to the motion control library to access
/// the drive functionality (like reading and restoring the position counter)
/// directly.
/// \param[in] hPump A valid pump handle
/// \retval 0 - No pump drive access possible
/// \retval 1 - Pump drive handle
/// \retval <0 - Error code - Error getting pump drive handle
///
LCP_Func dev_hdl LCP_CALL LCP_GetPumpDriveHandle(dev_hdl hPump);


//---------------------------------------------------------------------------
/// Query the value of the internal drive position counter.
/// \see LCP_RestoreDrivePosCnt()
/// \param[in] hPump Valid pump handle
/// \param[out] pPosCntValue Returns the actual value of the position
///                          counter
/// \return Error code - ERR_NOERR indicates success
///
LCP_Func long LCP_CALL LCP_GetDrivePosCnt(dev_hdl hPump, long *pPosCntValue);


//---------------------------------------------------------------------------
/// Restore internal hardware position counter value of pump drive.
/// The function restores the internal position counter value
/// saved with LCA_GetDrivePosCnt().
/// \note In many drives the actual position value is counted by a
/// quadrature decoder. This internal position counter value will get
/// lost, as soon as the device is switched off. In order to restore
/// this position counter value after power on, an application can
/// query the internal position counter value (LCA_GetDrivePosCnt()), store
/// it persistently into a configuration file and restore it later by calling
/// this function.
/// \see LCP_GetDrivePosCnt()
/// \param[in] hPump Valid pump handle
/// \param[in] PosCntValue The position counter value to restore
/// \return Error code - ERR_NOERR indicates success
///
LCP_Func long LCP_CALL LCP_RestoreDrivePosCnt(dev_hdl hPump, long PosCntValue);
/// \} lcp_drv


//===========================================================================
//                            PUMP VALVE
//
/// \addtogroup lcp_valve Pump Valve Control
/// Functions to access the optional valve that may be assigned to a certain
/// pump device.
/// \{
//===========================================================================

//--------------------------------------------------------------------------
/// Returns 1 if this pump has an valve assigned.
/// \param[in] hPump A valid pump handle
/// \retval 0 - No valve assigned to this pump
/// \retval 1 - Valve assigned to this pump
/// \retval <0 - Error code
///
LCP_Func long LCP_CALL LCP_HasValve(dev_hdl hPump);


//--------------------------------------------------------------------------
/// Returns the valve handle of a pump valve for the pump given in hPump
/// parameter.
/// \param[in] hPump A valid pump handle
/// \param[out] ValveHandle Handle to valve device or 0 if no valve with given
///             name exists
/// \retval -ERR_NODEV No device with the given name found
/// \retval ERR_NOERR indicates success
///
LCP_Func long LCP_CALL LCP_GetValveHandle(dev_hdl hPump,
	dev_hdl* ValveHandle);
/// \} lcp_valve


//===========================================================================
//                        AUXILIARY FUNCTIONS
//
/// \addtogroup lcp_common Auxiliary Functions
/// Common pump specific auxiliary functions.
/// \{
//===========================================================================

//---------------------------------------------------------------------------
/// Query name of specific pump.
/// Normally all pumps are named. That means each pump carries a
/// unique name.
/// \param[in]  hPump           Valid pump handle
/// \param[out] pNameStringBuf  Message buffer for storage of error message string
/// \param[in]  StringBufSize   Size of buffer pNameStringBuf in bytes
/// \return Error code - - ERR_NOERR indicates success
///
LCP_Func long LCP_CALL LCP_GetPumpName(dev_hdl  hPump,
                                        char    *pNameStringBuf,
                                        int      StringBufSize);

/// \} lcp_common

//===========================================================================
//                         TIME-TRIGGERED DOSAGE
//
/// \addtogroup lcp_tt Time-triggered dosage
/// Functions to configure and control time-triggered dosage.
/// Time trigger dosage is a feature that enables time-triggered start
/// of dosage processes. Each pump contains batch list. This list stores
/// single batch entries. A batch entry contains the complete configuration
/// for one single time-triggered dosage task. A batch entry contains\n
/// - Flow
/// - Volume
/// - Date and Time
/// - Cycles
/// - Period
/// The flow and volume values are the configuration for the dosage process
/// that is triggered at a certain time. The date and time value configures
/// The absolute date and time, when this dosage will be triggered the first
/// time. The number of cycles configures how often this dosage should be
/// repeated. A value of 0 indicates an infinite repetition. The period
/// configures the time between two consecutive dosages.
/// \{
//===========================================================================

//---------------------------------------------------------------------------
/// Query current number of entries in a pumps batch list.
/// \param[in]  hPump      Valid pump handle
/// \retval >= 0 Number of entries in batch list
/// \retval <0 Error code
///
LCP_Func long LCP_CALL LCP_GetBatchListSize(dev_hdl hPump);


//---------------------------------------------------------------------------
/// Deletes all entries from a pumps batch list
/// \param[in]  hPump      Valid pump handle
/// \return Error code - ERR_NOERR indicates success
///
LCP_Func long LCP_CALL LCP_ClearBatchList(dev_hdl hPump);


//---------------------------------------------------------------------------
/// This function returns a handle to a certain batch entry.
/// To access the parameters of a batch entry, a valid batch entry handle
/// is required. This handle can be obtained by this function
/// \param[in] hPump  Valid pump handle
/// \param[in] Index  Index into internal array of batch entries
/// \param[out] pBatchEntryHdl Handle to the batch entry indexed by Index
///                            param
/// \return Error code - ERR_NOERR indicates success
/// \retval -ERR_PARAM_RANGE - Index parameter is out of range (>= size
///                            of batch list)
/// \note An index does not always identify the same batch entry. The batch
///       entries a stored in a certain order and this order may change
///       during execution or after insertion/deletion of entries
///
LCP_Func long LCP_CALL LCP_GetBatchEntryAtIndex(dev_hdl        hPump,
                                                 unsigned long  Index,
                                                 dev_hdl       *pBatchEntryHdl);


//---------------------------------------------------------------------------
/// This function returns a handle to a certain batch entry.
/// The functions searches for an entry with a certain starting time. The time
/// value identifies the initial starting time.
/// \param[in] hPump  Valid pump handle
/// \param[in] Time Time value (\see LCP_MakeTime())
/// \param[out] pBatchEntryHdl Handle to the batch entry identified by Time
///                            param
/// \return Error code - ERR_NOERR indicates success
///
LCP_Func long LCP_CALL LCP_GetBatchEntry(dev_hdl        hPump,
                                          unsigned long  Time,
                                          dev_hdl       *pBatchEntryHdl);


//---------------------------------------------------------------------------
/// This function returns the configuration of a certain batch entry
/// \param[in] BatchEntryHdl  Valid batch entry handle
/// \param[out] pVolume Returns the volume value
/// \param[out] pFlow Returns the flow value
/// \param[out] pTime Returns the first start time of this batch entry
/// \param[out] pCycles Returns the number of cycles this batch entry should be
///                     executed
/// \param[out] pPeriodSecs Returns the period in seconds
/// \return Error code - ERR_NOERR indicates success
///
LCP_Func long LCP_CALL LCP_GetBatchEntryParam(dev_hdl       BatchEntryHdl,
                                               double        *pVolume,
                                               double        *pFlow,
                                               unsigned long *pTime,
                                               unsigned long *pCycles,
                                               unsigned long *pPeriodSecs);


//---------------------------------------------------------------------------
/// Adds a single entry to list of batch entries
/// \param[in] hPump  Valid pump handle
/// \param[in] Time   The time that triggers first start of this dosage
/// \param[in] Volume The volume to dose
/// \param[in] Flow   The flow rate to use for dosage
/// \param[in] Cycles The number of cycles to repeat this dosage. A value
///                   of 0 indicates an infinite repetition
/// \param[in] PeriodSecs The time in seconds between two consecutive
///                       execution of this dosage batch entry
/// \return Error code - ERR_NOERR indicates success
///
LCP_Func long LCP_CALL LCP_AddBatchEntry(dev_hdl         hPump,
                                          double          Volume,
                                          double          Flow,
                                          unsigned long   Time,
                                          unsigned long   Cycles,
                                          unsigned long   PeriodSecs);


//---------------------------------------------------------------------------
/// Remove batch entry from list of batch entries
/// \param[in] hPump  Valid pump handle
/// \param[in] Time   Indicates the starting time of the batch entry to be
///                   removed
/// \return Error code - ERR_NOERR indicates success
///
LCP_Func long LCP_CALL LCP_RemoveBatchEntry(dev_hdl       hPump,
                                             unsigned long Time);


//---------------------------------------------------------------------------
/// Start processing of batch list
/// \param[in] hPump  Valid pump handle of pump to start
/// \return Error code - ERR_NOERR indicates success
///
LCP_Func long LCP_CALL LCP_StartBatchProcessing(dev_hdl hPump);


//---------------------------------------------------------------------------
/// Stop processing of batch list
/// \param[in] hPump  Valid pump handle of pump to stop
/// \return Error code - ERR_NOERR indicates success
///
LCP_Func long LCP_CALL LCP_StopBatchProcessing(dev_hdl hPump);


//---------------------------------------------------------------------------
/// Check if batch processing is active
/// \param[in] hPump  Valid pump handle of pump to check
/// \retval 0  Batch processing is inactive
/// \retval >0 Batch processing is active
/// \retval <0 Error code - ERR_NOERR indicates success
///
LCP_Func long LCP_CALL LCP_IsBatchRunning(dev_hdl hPump);


//---------------------------------------------------------------------------
/// Returns time of next dosage event
/// \note The function only returns a valid value if the batch is running
///       (\see LCP_IsBatchRunning()). If the batch is not running, then
///       the time value returned is undefined.
/// \param[in] hPump  Valid pump handle of pump to check
/// \retval >=0 Time value of next dosage event
/// \retval <0 Error code
///
LCP_Func long LCP_CALL LCP_GetNextDosageTime(dev_hdl hPump);
/// \} lcp_tt
/// \} lcp_api
//---------------------------------------------------------------------------
#endif // labbCAN_Pump_APIH

